package org.projectopen.timesheet;

/*
 * Copyright (C) 2010 ]project-open[ 
 *
 * This program is free software. You can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2 of the License, or (at your option)
 * any later version. This program is distributed in the
 * hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * @author Frank Bergmann (frank.bergmann@project-open.com)
 * 
 */

import java.util.Iterator;
import java.util.List;

import org.projectopen.treetable.AbstractTreeTableModel;
import org.projectopen.treetable.TreeTableModel;

@SuppressWarnings("unchecked")

/**
 * Data model for JTreeTable.
 * This class basically delegates getValueAt and setValueAt calls
 * from the JTreeTable to the REST interface, apart from interfacing
 * with the project hierarchy.
 */
public class ProjectTreeTableModel extends AbstractTreeTableModel implements TreeTableModel {

    static protected String[] cNames = {"Name", "Hours", "Note"};
	static protected Class[] cTypes = {TreeTableModel.class, Float.class, String.class};
	
	public ProjectTreeTableModel() {
		super(new ProjopProject());
		System.out.println("ProjectTreeTableModel.ProjectTreeTableModel");
		initialize();
	}
	
	private void initialize() {
		System.out.println("ProjectTreeTableModel.initialize");

        // create a dummy project at the top of the tree
        root = new ProjopProject();
        ((ProjopProject)root).set("project_name", "]project-open[ Projects");

		// here we will store the last parents of each level,
		// in order to create a hierarchy of projects.
        ProjopProject[] node = new ProjopProject[10];
        node[0] = ((ProjopProject)root);

        // count the level of depth of the projects.
        int level;

        // get the list of projects to which the current user can
        // log hours.
        List projects = RESTClient.projopMyTimesheetProjects();

        // go through the list and add them to the project hierarchy
        Iterator itr = projects.iterator(); 
        while(itr.hasNext()) {

        	// get the next project
            ProjopProject p = (ProjopProject) itr.next(); 

            // Main-Projects in ]po[ have level=0, but here they are
            // level=1, because the are located below the ficticous
            // "]project-open[ Projects" node which occupies level=0
            String levelString = p.get("level");            
            level = 1 + Integer.parseInt(levelString.trim());
            
            // Set the parent as the project one above in the hierarchy
            p.setParent(node[level-1]);

            // set the new projectNode as the parent for the next projects..
            node[level] = p;
 
            // add this project to the list of parent's children
            node[level-1].getChildProjects().add(p);
        }
	    
		System.out.println("ProjectTreeTableModel.initialize end");
	}
	
	public Class getColumnClass(int column) {
		return cTypes[column];
	}

	public int getColumnCount() {
		return cTypes.length;
	}

	public String getColumnName(int column) {
		return cNames[column];
	}

	public Object getValueAt(Object node, int column) {
		
		int projectId = ((ProjopProject)node).getObjectId();
		ProjopHour h = RESTClient.projopMyHours(projectId);
    
	    switch(column) {
	    	case 0:	{
	    		// return a string with the name of the project.
	    		if (projectId == 0) { return ""; }
	    		return node.toString();
	    	}
	    	case 1:	{
	    		// return the number of hours logged
	    		if (h != null) {
	    			String hoursString = h.get("hours");
	    			if ("" != hoursString) {
	    				return new Double(hoursString);
	    			}
	    		}
	    		return null;
	    	}
	    	case 2:	{
	    		if (h != null) {
	    			return h.get("note");
	    		}
	    		return null;
	    	}
	    }
	    return null;
	}

    public void setValueAt(Object aValue, Object node, int column) {

    	int projectId = ((ProjopProject)node).getObjectId();
		if (projectId == 0) { return; }
    
	    switch(column) {
	    	case 0:	{
	    		// Don't change the project name...
	    		break; 
	    	}
	    	case 1:	{
	    		// store the new number of hours
	    		RESTClient.setMyHours("hours", projectId, aValue);
	    		break; 
	    	}
	    	case 2:	{
	    		// store the new comment about the logged hours
	    		if (aValue == null) { aValue = ""; }
	    		aValue = ProjopObject.htmlEncode(aValue.toString());
	    		RESTClient.setMyHours("note", projectId, aValue);
	    		break; 
	    	}
	    }    	
    }
	
	
	@Override
	public boolean isCellEditable(Object node, int column) {
	    switch(column) {
    		case 0:	return true;
    		case 1:	return true;
    		case 2:	return true;
	    }
	    return false;
	}

	@Override
	public Object getChild(Object parent, int index) {
		Object child = ((ProjopProject) parent).getChildProjects().get(index); 
		return child;
	}

    public Object[] getChildren(Object node) {
    	Object[] children = ((ProjopProject)node).getChildProjects().toArray();
       	return children; 
    }
	
	@Override
	public int getChildCount(Object parent) {
		int count = ((ProjopProject) parent).getChildProjects().size();
		return count;
	}
	
	public boolean isLeaf(Object node) {
		int count = ((ProjopProject) node).getChildProjects().size();
		boolean r = (count == 0);
		return r;
	}

}
